#include <QFileDialog>
#include <QDesktopServices>
#include <QListView>
#include <QBoxLayout>
#include <QPushButton>
#ifdef Q_WS_MAEMO_5
#include <QMaemo5InformationBox>
#include <QDBusConnection>
#include <QDBusMessage>
#include <QEvent>
#else
#include <QMessageBox>
#endif
#include <Phonon/SeekSlider>
#include <Phonon/MediaObject>
#include <Phonon/AudioOutput>

#ifdef Q_WS_MAEMO_5
#include <mce/mode-names.h>
#include <mce/dbus-names.h>
#endif

#include "mainwindow.h"
#include "musiclistmodel.h"
#include "phononsourcedelegate.h"

MainWindow::MainWindow ( QWidget* parent, Qt::WindowFlags flags ) : QMainWindow(parent,flags), m_model(new MusicListModel(this)),m_layout(0L) {
  m_view = new QListView(this);
  m_view->setModel(m_model);
  m_load_collection = new QPushButton("load",this);
#ifndef Q_WS_MAEMO_5
  m_mode_toggler = new QPushButton("rotate",this);
#else
  m_mode_toggler = 0L;
#endif
  m_landscape_mode = false;
  m_phonon_delegate = new PhononSourceDelegate(m_view);

  m_view->setItemDelegate(m_phonon_delegate);

  m_mediaobject = new Phonon::MediaObject(this);
  m_slider = new Phonon::SeekSlider(this);
  Phonon::AudioOutput *device = new Phonon::AudioOutput;
  Phonon::createPath(m_mediaobject, device);

  m_slider->setMediaObject(m_mediaobject);

  connect(m_load_collection,SIGNAL(clicked()),this,SLOT(setCollection()));
  connect(m_mode_toggler,SIGNAL(clicked()),this,SLOT(toggleLandscape()));
  connect(m_view,SIGNAL(clicked(QModelIndex)),this,SLOT(play(QModelIndex)));

  connect(m_mediaobject,SIGNAL(metaDataChanged()),this,SLOT(track_playing()));
  connect(m_mediaobject,SIGNAL(finished()),this,SLOT(next()));
  setCentralWidget(new QWidget(this));
  setCollection();
#ifdef Q_WS_MAEMO_5
  QDBusConnection::systemBus().connect(QString(), MCE_SIGNAL_PATH, MCE_SIGNAL_IF,
                                       MCE_DEVICE_ORIENTATION_SIG,
                                       this,
                                       SLOT(orientationChanged(QString)));
#endif
  setPortrait();
}


void MainWindow::setLandscape() {
  QBoxLayout* lay = new QHBoxLayout();
  m_landscape_mode=true;
  lay->addWidget(m_view);
  QVBoxLayout* lay2 = new QVBoxLayout();
  lay2->addWidget(m_load_collection);
  if(m_mode_toggler) {
    lay2->addWidget(m_mode_toggler);
  }
  lay2->addWidget(m_slider);
  lay->addLayout(lay2);
  m_phonon_delegate->setPortraitMode(false);
  delete centralWidget()->layout();
 #ifdef Q_WS_MAEMO_5
  setAttribute(Qt::WA_Maemo5ForceLandscapeOrientation, true);
 #endif
  centralWidget()->setLayout(lay);
  m_layout = lay;
}


void MainWindow::setPortrait() {
  QBoxLayout* lay = new QVBoxLayout();
  m_landscape_mode = false;
  lay->addWidget(m_view);
  lay->addWidget(m_load_collection);
  if(m_mode_toggler) {
    lay->addWidget(m_mode_toggler);
  }
  lay->addWidget(m_slider);
  m_phonon_delegate->setPortraitMode(true);
  delete centralWidget()->layout();
 #ifdef Q_WS_MAEMO_5
  setAttribute(Qt::WA_Maemo5ForcePortraitOrientation, true);
 #endif
  centralWidget()->setLayout(lay);
  m_layout = lay;
}


void MainWindow::setCollection() {
  QStringList files = QFileDialog::getOpenFileNames(this, tr("Select Music Files"),
    QDesktopServices::storageLocation(QDesktopServices::MusicLocation));
  m_model->loadFiles(files);
}


void MainWindow::toggleLandscape() {
  if(m_landscape_mode==true) {
    setPortrait();
    return;
  } else if(m_landscape_mode == false) {
    setLandscape();
    return;
  }
}



void MainWindow::play (const QModelIndex& index) {
  if(!index.isValid()) {
    return;
  }
  QVariant v = m_model->data(index,MusicListModel::MediaSource);
  Phonon::MediaSource media = v.value<Phonon::MediaSource>();
  m_mediaobject->setCurrentSource(media);
  m_mediaobject->play();
  next_track = m_model->index(index.row()+1,index.column());
}

void MainWindow::track_playing() {
  QString title, artist;
  if(!m_mediaobject->metaData(Phonon::TitleMetaData).isEmpty())
    title = m_mediaobject->metaData(Phonon::TitleMetaData).front();
  if(!m_mediaobject->metaData(Phonon::ArtistMetaData).isEmpty())
    artist = m_mediaobject->metaData(Phonon::ArtistMetaData).front();
#ifdef Q_WS_MAEMO_5
  QMaemo5InformationBox::information(this, QString("<b>Playing</b> %1 by %2").arg(title).arg(artist),
                                            QMaemo5InformationBox::DefaultTimeout);
#else
  QMessageBox::information(this,QString("playing"), QString("<b>playing</b> %1 by %2").arg(title).arg(artist));
#endif
}

void MainWindow::next() {
  if(!next_track.isValid()) {
    next_track = m_model->index(0,0);
  }
  play(next_track);
}

void MainWindow::play() {
  next_track = m_model->index(0,0);
  next();
}

void MainWindow::orientationChanged (const QString& neworientation) {
#ifdef Q_WS_MAEMO_5
  if (neworientation == QLatin1String(MCE_ORIENTATION_PORTRAIT)) {
    setPortrait();
  } else {
    setLandscape();
  }
#else
  Q_UNUSED(neworientation);
  toggleLandscape();
#endif

}


bool MainWindow::event (QEvent* event) {
#ifdef Q_WS_MAEMO_5
  switch (event->type()) {
    case QEvent::WindowActivate:
      QDBusConnection::systemBus().call(
           QDBusMessage::createMethodCall(MCE_SERVICE, MCE_REQUEST_PATH,
                                          MCE_REQUEST_IF,
                                          MCE_ACCELEROMETER_ENABLE_REQ));
      break;
    case QEvent::WindowDeactivate:
      QDBusConnection::systemBus().call(
           QDBusMessage::createMethodCall(MCE_SERVICE, MCE_REQUEST_PATH,
                                          MCE_REQUEST_IF,
                                          MCE_ACCELEROMETER_DISABLE_REQ));
      break;
    default:
      break;
  }
#endif
  return QMainWindow::event (event);
}

