#include <QPainter>
#include <QApplication>
#include <QUrl>
#include <QAbstractItemView>

#include <Phonon/MediaObject>
#include <Phonon/MediaSource>

#include "phononsourcedelegate.h"
#include "musiclistmodel.h"

PhononSourceDelegate::PhononSourceDelegate (QAbstractItemView* parent) : QStyledItemDelegate (parent), parent_view(parent) {
  m_portrait=false;
  m_metadatagatherer = new Phonon::MediaObject(this);
  connect(m_metadatagatherer,SIGNAL(metaDataChanged()),this,SLOT(changed()));
}

void PhononSourceDelegate::paint (QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const {
  QVariant v =  index.data(MusicListModel::MediaSource);
  Phonon::MediaSource media;;
  if(v.canConvert<Phonon::MediaSource>()) {
    media = v.value<Phonon::MediaSource>();
    if(!m_data.contains(media)) {
      m_metadatagatherer->setCurrentSource(media);
    }
  }
  QStyle* style = QApplication::style();
  if(m_portrait) {
    if(m_data.contains(media)) {
      painter->drawText(option.rect,m_data[media].title);
    } else {
      painter->drawText(option.rect,media.url().toString());
    }
  } else {
    if(m_data.contains(media)) {
      painter->drawText(option.rect,m_data[media].artist);
    } else {
      painter->drawText(option.rect,media.url().toString());
    }
  }
}


void PhononSourceDelegate::changed() {
  Phonon::MediaSource media = m_metadatagatherer->currentSource();
  MetaData m;
  m.title = m_metadatagatherer->metaData(Phonon::TitleMetaData).first();
  m.album = m_metadatagatherer->metaData(Phonon::AlbumMetaData).first();
  m.artist = m_metadatagatherer->metaData(Phonon::ArtistMetaData).first();
  m_data[media] = m;
}


void PhononSourceDelegate::setPortraitMode (bool portrait) {
  m_portrait = portrait;
}
